"""
Parsel lets you extract text from XML/HTML documents using XPath
or CSS selectors
"""

__author__ = 'Scrapy project'
__email__ = 'info@scrapy.org'
__version__ = '0.9.0'

from parsel.unified import Selector, SelectorList  # NOQA
